/*
 * Lange Nacht der Wissenschaften 2013
 * 
 * Elektronischer Wrfel
 * 
 * 5.6.2013 Sven Tschirley
 *
 *  main:
 *  +------------------------------------+           +---------------------------------------+
 *  | Initialsierrung von allem          |           | ISR Taste                             |
 *  +----------------+-------------------+           |---------------------------------------|
 *                   |                               | Wenn powerDown aktiv dann ausschalten |
 *  +----------------v-------------------+           | sonst neuen Wurf anmelden             |
 *  | LEDs testen                        |           +---------------------------------------+
 *  +----------------+-------------------+
 *                   |                               +---------------------------------------+
 *  +----------------v-------------------+           | ISR Timer                             |
    | Zufallsgenerator initialisieren    |           |---------------------------------------|
 *  +----------------+-------------------+           | Wenn Wartezeit verstrichen, dann wird |
 *                   |                               | powerDown aktiviert                   |
 *  +--------------->+------+                        +---------------------------------------+
 *  |                       |
 *  |     +-----------------v--------------------+
 *  |     | Zufallszahl erzeugen ( verdeckt )    |
 *  |     +-----------------+--------------------+
 *  |                       |
 *  |                       |
 *  |     +-----------------v>-------------------+
 *  |     | Wenn Taste gedrueckt, dann wuerfel   |
 *  |     | rollen und Zufallszahl erzeugen      |
 *  |     |                                      |
 *  |     +-----------------+--------------------+
 *  |                       |
 *  |     +-----------------v--------------------+
 *  |     | Wenn kein powerDown vorliegt,        |
 *  |     | zufallszahl anzeigen, sonst nix      |
 *  |     +-----------------+--------------------+
 *  |                       |
 *  |                       |
 *  |                       |
 *  +-----------------------+
 *
 */

#include <stdbool.h>
#include <stdlib.h>
#include <msp430g2231.h>

/****************************************************/

#define LED_BIT   (LED1|LED2|LED3|LED4|LED5|LED6|LED7)
#define LED_AUS	  !(LED_BIT)
#define LED1	  BIT0
#define	LED2	  BIT1
#define	LED3	  BIT2
#define LED4	  BIT4
#define LED5	  BIT5
#define LED6      BIT6
#define LED7      BIT7

#define TRUE      0x01
#define FALSE     0x00


#define LAMPTEST_DELAY 40000

#define BLITZDAUER 10000
// Timer0 ticks
#define TICKS_PER_SEC (1000 / TACCR0)    // 1MHz clock

/****************************************************/
volatile char mode = 0;
volatile unsigned int  timerCounter = 0;
volatile unsigned int  intTimerCounter = 0;
volatile unsigned char augenzahl = 0;
volatile unsigned char anzeige   = 0;

volatile unsigned int  lfsr = 0xACE1;
unsigned char          taste = FALSE;
unsigned char          powerDown = FALSE;
/****************************************************/
/* 
 * Funktionsprototypen
 */
void          ConfigWDT( void );
void          ConfigClocks( void );
void          ConfigLEDs( void );
void          rollen ( void );
unsigned char wuerfeln( void );

/*
 * Konfiguration des Watchdogs -- einfach ausschalten,
 * den brauchen wir nicht. Der Controller arbeitet auch
 * mit fast garkeiner Batteriespannung
 */
void ConfigWDT( void )
{
  WDTCTL = WDTPW + WDTHOLD; // Stop WDT
}

/*
 * Konfigurieren der Takterzeugung
 */
void ConfigClocks( void )
{
  BCSCTL1 = CALBC1_1MHZ; 		            // Set range
  DCOCTL = CALDCO_1MHZ;  		            // Set DCO step + modulation
  BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  IFG1 &= ~OFIFG;                           // Clear OSCFault flag
  BCSCTL2 |= SELM_0 + DIVM_3 + DIVS_3;      // MCLK = DCO/8, SMCLK = DCO/8	
}

/*
 * Konfigurieren des Port1 mit den LEDs
 * Alle LEDs sind Ausgaenge
 * P1.3 ist die Taste und damit ein Eingang
 */
void ConfigPort1( void )
{
  P1DIR = 0;
  /* Initialisierung der LEDs                                   */ 
  P1DIR |= LED_BIT;     // Alle LED pins als Ausgang schalten
  P1OUT &= LED_BIT;  	// Alle LEDs ausschalten
  	
  P1IE  |= 0x08;        // P1.3 interrupt enabled
  P1IES |= 0x08;        // P1.3 Hi/lo edge
  P1IFG &= ~0x08;       // P1.3 IFG cleared
}

/*
 * Konfigurieren des Timers
 * Takt 1MHZ, CCR0 = 0x1000 liefert T = 2,56ms
 */
void ConfigTimerA0( void )
{
  CCTL1 = OUTMOD_0;
  CCR1  = 32;
  CCTL0 = CCIE;
  CCR0  = 0x1000;
  TACTL = TASSEL_2 + MC_1; 	
}

/****************************************************/
/*
 * Interrupt Service Routinen (ISR)
 */
/*
 * TIMER A0 ISR
 * 
 * Der Timer zaehlt bis ca 45sec, dann wird powerDown ausgeloest.
 */
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A_ISR( void )
{
  timerCounter++;
  if ( timerCounter == 0x400)
    {
	  powerDown = TRUE;
    }
}

/*
 * Port 1 interrupt service routine
 * 
 * Nach betaetigen der Taste wird geschaut, ob der powerDown
 * angeschaltet ist. Wenn das so ist, dann wird das ausgeschaltet,
 * damit die LEDs den alten Zustand anzeigen koennen.
 * Wenn wir im aktiven Wuerfeln sind, dann wird ein neuer
 * Wurf angemeldet. Gemacht wird der Wurf in der Hauptschleife.
 */
#pragma vector=PORT1_VECTOR
__interrupt void Port_1_ISR( void )
{
  P1IFG &= ~0x08;             // P1.4 IFG cleared
  if ( powerDown == TRUE )
    {
	  powerDown = FALSE;
	  timerCounter = 0;
    }
  else
    {
	  powerDown = FALSE;
	  taste = TRUE;
    }
}

/*
 * nicht verwentete Interrupts werden als leere Funktionen
 * implementiert, damit nicht irgendwas eingefangen wird.
 * Das sollte nich passieren, da die alle ausgeschaltet sind,
 * aber man kann ja nie wissen...
 */
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR( void ){}

#pragma vector=PORT2_VECTOR
__interrupt void Port_2_ISR( void ){}

#pragma vector=NMI_VECTOR
__interrupt void NMI_ISR( void ){}

#pragma vector=TIMERA1_VECTOR
__interrupt void Timer_A1_ISR( void ){}

#pragma vector=USI_VECTOR
__interrupt void USI_ISR( void ){}

#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR( void ){}


/****************************************************/
/* Relevante Funktionen fr den Programmablauf	    */

/*
 * Testfunktion zum Lampentest, es werden alle LEDs
 * einmal eingeschaltet
 */
void LampTest( void )
{
  P1OUT = LED1;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED2;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED3;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED4;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED5;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED6;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
  P1OUT = LED7;  
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED_AUS;
}

/* 
 * Erzeugen eines Blinkspiels wenn der Wrfel rollt
 */
 void rollen ( void )
{
  P1OUT = LED_AUS;
  _delay_cycles( LAMPTEST_DELAY );
  P1OUT = LED5;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7|LED1;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7|LED1;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7|LED1|LED3;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7|LED1|LED3|LED4;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED5|LED7|LED1|LED3|LED4|LED2;
  _delay_cycles( BLITZDAUER );
  P1OUT = LED_AUS;
}

/*
 * Hier wird gewrfelt. Es gibt hierfuer eine eingebaute
 * Zufallsfunktion, die hier verwendet wird.
 */
unsigned char wuerfeln ( void )
{
  do                        // in der Schleife fangen wir ab, das auch eine 
  	{                       // NULL erwrfelt werden kann. Einfach nochmal machen
  	  augenzahl = rand()& 0x07;
  	} 
  while ( (augenzahl == 0) || (augenzahl == 0x07) );
  return( augenzahl );
}

/*
 * Das ist das Hauptprogramm.
 * Was da genau passiert, steht in den Kommentraten im Quelltext.
 */
int main(void)
{
  /* Aufruf der Initialisierungsfunktion fuer den Controller     */
  ConfigWDT();		// Watchdog
  ConfigClocks();	// Takt
  ConfigPort1();	// LEDs
  ConfigTimerA0();	// Timer      	
  
  /* Einschalten aller Interrupts im Controller, die vorher     */
  /* initialisiert wurden                                       */
  _BIS_SR( GIE );
  /* Einmal eine Leuchtsequenz durchlaufen zum Selbsttest       */
  LampTest();
  /* Zufallsgenerator initialisieren                            */
  srand( lfsr );
  /* Es startet eine Endlosschleife, die der Controller         */
  /* abarbeitet, bis ihm der Strom ausgeht.                     */
  while(1)
    { 
      /* gewuerfelt wird immer bei jedem Schleifendurchlauf     */
      augenzahl = wuerfeln();
      /* Wenn die Taste gedrueckt wurde, dann rollt der Wuerfel */
      /* am Ende wird die augenzahl angezeigt*/
      if ( taste == TRUE )
      	{
    	  taste   = FALSE;
    	  rollen();
    	  anzeige = augenzahl;
      	}
      /*
       * Wenn wir im powerDown Modus sind, ist alles dunkel, sonst
       * wird die LED Anzeige aus dem Wert von augenzahl erzeugt
       */
      if ( powerDown == FALSE )
        {
    	  switch( anzeige )
    	    {
    	  	  case 0x01:          // eins
    	        P1OUT = LED_AUS;
                P1OUT = LED4;
                break;
              case 0x02:			// zwei
                P1OUT = LED_AUS;
                P1OUT = LED1|LED7;
                break;
              case 0x03:			// drei
                P1OUT = LED_AUS;
                P1OUT = LED1|LED4|LED7;
                break;
              case 0x04:			// vier
                P1OUT = LED_AUS;
                P1OUT = LED1|LED3|LED5|LED7;
                break;
              case 0x05:			// fnf
                P1OUT = LED_AUS;
                P1OUT = LED1|LED3|LED4|LED5|LED7;
                break;
              case 0x06:            // sechs
                P1OUT = LED_AUS;
                P1OUT = LED1|LED2|LED3|LED5|LED6|LED7;
                break;
              default:
                P1OUT = LED_AUS;			// alles aus
                break;
    	    }  // end of switch ...
        }
      else
        {
    	  P1OUT = LED_AUS;
        }
    }          // end of while (1 )
}
